# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ChatStreamRequestPromptTruncation(str, enum.Enum):
    """
    Defaults to `AUTO` when `connectors` are specified and `OFF` in all other cases.

    Dictates how the prompt will be constructed.

    With `prompt_truncation` set to "AUTO", some elements from `chat_history` and `documents` will be dropped in an attempt to construct a prompt that fits within the model's context length limit.

    With `prompt_truncation` set to "OFF", no elements will be dropped. If the sum of the inputs exceeds the model's context length limit, a `TooManyTokens` error will be returned.
    """

    OFF = "OFF"
    AUTO = "AUTO"

    def visit(self, off: typing.Callable[[], T_Result], auto: typing.Callable[[], T_Result]) -> T_Result:
        if self is ChatStreamRequestPromptTruncation.OFF:
            return off()
        if self is ChatStreamRequestPromptTruncation.AUTO:
            return auto()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime
from .api_meta import ApiMeta
from .embed_job_status import EmbedJobStatus
from .embed_job_truncate import EmbedJobTruncate

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class EmbedJob(pydantic.BaseModel):
    job_id: str = pydantic.Field(description="ID of the embed job")
    name: typing.Optional[str] = pydantic.Field(default=None, description="The name of the embed job")
    status: EmbedJobStatus = pydantic.Field(description="The status of the embed job")
    created_at: dt.datetime = pydantic.Field(description="The creation date of the embed job")
    input_dataset_id: str = pydantic.Field(description="ID of the input dataset")
    output_dataset_id: typing.Optional[str] = pydantic.Field(
        default=None, description="ID of the resulting output dataset"
    )
    model: str = pydantic.Field(description="ID of the model used to embed")
    truncate: EmbedJobTruncate = pydantic.Field(description="The truncation option used")
    meta: typing.Optional[ApiMeta] = None

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .api_meta import ApiMeta
from .api_meta_api_version import ApiMetaApiVersion
from .api_meta_billed_units import ApiMetaBilledUnits
from .auth_token_type import AuthTokenType
from .chat_citation import ChatCitation
from .chat_citation_generation_event import ChatCitationGenerationEvent
from .chat_connector import ChatConnector
from .chat_document import ChatDocument
from .chat_message import ChatMessage
from .chat_message_role import ChatMessageRole
from .chat_request_citation_quality import ChatRequestCitationQuality
from .chat_request_prompt_override import ChatRequestPromptOverride
from .chat_request_prompt_truncation import ChatRequestPromptTruncation
from .chat_request_search_options import ChatRequestSearchOptions
from .chat_request_tool_results_item import ChatRequestToolResultsItem
from .chat_search_queries_generation_event import ChatSearchQueriesGenerationEvent
from .chat_search_query import ChatSearchQuery
from .chat_search_result import ChatSearchResult
from .chat_search_result_connector import ChatSearchResultConnector
from .chat_search_results_event import ChatSearchResultsEvent
from .chat_stream_end_event import ChatStreamEndEvent
from .chat_stream_end_event_finish_reason import ChatStreamEndEventFinishReason
from .chat_stream_event import ChatStreamEvent
from .chat_stream_request_citation_quality import ChatStreamRequestCitationQuality
from .chat_stream_request_prompt_override import ChatStreamRequestPromptOverride
from .chat_stream_request_prompt_truncation import ChatStreamRequestPromptTruncation
from .chat_stream_request_search_options import ChatStreamRequestSearchOptions
from .chat_stream_request_tool_results_item import ChatStreamRequestToolResultsItem
from .chat_stream_start_event import ChatStreamStartEvent
from .chat_text_generation_event import ChatTextGenerationEvent
from .chat_tool_inputs_generation_event import ChatToolInputsGenerationEvent
from .classify_example import ClassifyExample
from .classify_request_truncate import ClassifyRequestTruncate
from .classify_response import ClassifyResponse
from .classify_response_classifications_item import ClassifyResponseClassificationsItem
from .classify_response_classifications_item_classification_type import (
    ClassifyResponseClassificationsItemClassificationType,
)
from .classify_response_classifications_item_labels_value import ClassifyResponseClassificationsItemLabelsValue
from .compatible_endpoint import CompatibleEndpoint
from .connector import Connector
from .connector_auth_status import ConnectorAuthStatus
from .connector_o_auth import ConnectorOAuth
from .create_connector_o_auth import CreateConnectorOAuth
from .create_connector_response import CreateConnectorResponse
from .create_connector_service_auth import CreateConnectorServiceAuth
from .create_embed_job_response import CreateEmbedJobResponse
from .dataset import Dataset
from .dataset_part import DatasetPart
from .dataset_type import DatasetType
from .dataset_validation_status import DatasetValidationStatus
from .delete_connector_response import DeleteConnectorResponse
from .detokenize_response import DetokenizeResponse
from .embed_by_type_response import EmbedByTypeResponse
from .embed_by_type_response_embeddings import EmbedByTypeResponseEmbeddings
from .embed_floats_response import EmbedFloatsResponse
from .embed_input_type import EmbedInputType
from .embed_job import EmbedJob
from .embed_job_status import EmbedJobStatus
from .embed_job_truncate import EmbedJobTruncate
from .embed_request_embedding_types_item import EmbedRequestEmbeddingTypesItem
from .embed_request_truncate import EmbedRequestTruncate
from .embed_response import EmbedResponse, EmbedResponse_EmbeddingsByType, EmbedResponse_EmbeddingsFloats
from .finish_reason import FinishReason
from .generate_request_return_likelihoods import GenerateRequestReturnLikelihoods
from .generate_request_truncate import GenerateRequestTruncate
from .generate_stream_end import GenerateStreamEnd
from .generate_stream_end_response import GenerateStreamEndResponse
from .generate_stream_error import GenerateStreamError
from .generate_stream_event import GenerateStreamEvent
from .generate_stream_request_return_likelihoods import GenerateStreamRequestReturnLikelihoods
from .generate_stream_request_truncate import GenerateStreamRequestTruncate
from .generate_stream_text import GenerateStreamText
from .generate_streamed_response import (
    GenerateStreamedResponse,
    GenerateStreamedResponse_StreamEnd,
    GenerateStreamedResponse_StreamError,
    GenerateStreamedResponse_TextGeneration,
)
from .generation import Generation
from .get_connector_response import GetConnectorResponse
from .list_connectors_response import ListConnectorsResponse
from .list_embed_job_response import ListEmbedJobResponse
from .list_models_response import ListModelsResponse
from .model import Model
from .non_streamed_chat_response import NonStreamedChatResponse
from .o_auth_authorize_response import OAuthAuthorizeResponse
from .parse_info import ParseInfo
from .rerank_request_documents_item import RerankRequestDocumentsItem
from .rerank_request_documents_item_text import RerankRequestDocumentsItemText
from .rerank_response import RerankResponse
from .rerank_response_results_item import RerankResponseResultsItem
from .rerank_response_results_item_document import RerankResponseResultsItemDocument
from .single_generation import SingleGeneration
from .single_generation_in_stream import SingleGenerationInStream
from .single_generation_token_likelihoods_item import SingleGenerationTokenLikelihoodsItem
from .streamed_chat_response import (
    StreamedChatResponse,
    StreamedChatResponse_CitationGeneration,
    StreamedChatResponse_SearchQueriesGeneration,
    StreamedChatResponse_SearchResults,
    StreamedChatResponse_StreamEnd,
    StreamedChatResponse_StreamStart,
    StreamedChatResponse_TextGeneration,
    StreamedChatResponse_ToolInputsGeneration,
)
from .summarize_request_extractiveness import SummarizeRequestExtractiveness
from .summarize_request_format import SummarizeRequestFormat
from .summarize_request_length import SummarizeRequestLength
from .summarize_response import SummarizeResponse
from .tokenize_response import TokenizeResponse
from .tool import Tool
from .tool_input import ToolInput
from .tool_parameter_definitions_value import ToolParameterDefinitionsValue
from .update_connector_response import UpdateConnectorResponse

__all__ = [
    "ApiMeta",
    "ApiMetaApiVersion",
    "ApiMetaBilledUnits",
    "AuthTokenType",
    "ChatCitation",
    "ChatCitationGenerationEvent",
    "ChatConnector",
    "ChatDocument",
    "ChatMessage",
    "ChatMessageRole",
    "ChatRequestCitationQuality",
    "ChatRequestPromptOverride",
    "ChatRequestPromptTruncation",
    "ChatRequestSearchOptions",
    "ChatRequestToolResultsItem",
    "ChatSearchQueriesGenerationEvent",
    "ChatSearchQuery",
    "ChatSearchResult",
    "ChatSearchResultConnector",
    "ChatSearchResultsEvent",
    "ChatStreamEndEvent",
    "ChatStreamEndEventFinishReason",
    "ChatStreamEvent",
    "ChatStreamRequestCitationQuality",
    "ChatStreamRequestPromptOverride",
    "ChatStreamRequestPromptTruncation",
    "ChatStreamRequestSearchOptions",
    "ChatStreamRequestToolResultsItem",
    "ChatStreamStartEvent",
    "ChatTextGenerationEvent",
    "ChatToolInputsGenerationEvent",
    "ClassifyExample",
    "ClassifyRequestTruncate",
    "ClassifyResponse",
    "ClassifyResponseClassificationsItem",
    "ClassifyResponseClassificationsItemClassificationType",
    "ClassifyResponseClassificationsItemLabelsValue",
    "CompatibleEndpoint",
    "Connector",
    "ConnectorAuthStatus",
    "ConnectorOAuth",
    "CreateConnectorOAuth",
    "CreateConnectorResponse",
    "CreateConnectorServiceAuth",
    "CreateEmbedJobResponse",
    "Dataset",
    "DatasetPart",
    "DatasetType",
    "DatasetValidationStatus",
    "DeleteConnectorResponse",
    "DetokenizeResponse",
    "EmbedByTypeResponse",
    "EmbedByTypeResponseEmbeddings",
    "EmbedFloatsResponse",
    "EmbedInputType",
    "EmbedJob",
    "EmbedJobStatus",
    "EmbedJobTruncate",
    "EmbedRequestEmbeddingTypesItem",
    "EmbedRequestTruncate",
    "EmbedResponse",
    "EmbedResponse_EmbeddingsByType",
    "EmbedResponse_EmbeddingsFloats",
    "FinishReason",
    "GenerateRequestReturnLikelihoods",
    "GenerateRequestTruncate",
    "GenerateStreamEnd",
    "GenerateStreamEndResponse",
    "GenerateStreamError",
    "GenerateStreamEvent",
    "GenerateStreamRequestReturnLikelihoods",
    "GenerateStreamRequestTruncate",
    "GenerateStreamText",
    "GenerateStreamedResponse",
    "GenerateStreamedResponse_StreamEnd",
    "GenerateStreamedResponse_StreamError",
    "GenerateStreamedResponse_TextGeneration",
    "Generation",
    "GetConnectorResponse",
    "ListConnectorsResponse",
    "ListEmbedJobResponse",
    "ListModelsResponse",
    "Model",
    "NonStreamedChatResponse",
    "OAuthAuthorizeResponse",
    "ParseInfo",
    "RerankRequestDocumentsItem",
    "RerankRequestDocumentsItemText",
    "RerankResponse",
    "RerankResponseResultsItem",
    "RerankResponseResultsItemDocument",
    "SingleGeneration",
    "SingleGenerationInStream",
    "SingleGenerationTokenLikelihoodsItem",
    "StreamedChatResponse",
    "StreamedChatResponse_CitationGeneration",
    "StreamedChatResponse_SearchQueriesGeneration",
    "StreamedChatResponse_SearchResults",
    "StreamedChatResponse_StreamEnd",
    "StreamedChatResponse_StreamStart",
    "StreamedChatResponse_TextGeneration",
    "StreamedChatResponse_ToolInputsGeneration",
    "SummarizeRequestExtractiveness",
    "SummarizeRequestFormat",
    "SummarizeRequestLength",
    "SummarizeResponse",
    "TokenizeResponse",
    "Tool",
    "ToolInput",
    "ToolParameterDefinitionsValue",
    "UpdateConnectorResponse",
]



# This file was auto-generated by Fern from our API Definition.

import typing

try:
    from typing import NotRequired  # type: ignore
except ImportError:
    from typing_extensions import NotRequired  # type: ignore


class RequestOptions(typing.TypedDict):
    """
    Additional options for request-specific configuration when calling APIs via the SDK.
    This is used primarily as an optional final parameter for service functions.

    Attributes:
        - timeout_in_seconds: int. The number of seconds to await an API call before timing out.

        - additional_headers: typing.Dict[str, typing.Any]. A dictionary containing additional parameters to spread into the request's header dict

        - additional_query_parameters: typing.Dict[str, typing.Any]. A dictionary containing additional parameters to spread into the request's query parameters dict

        - additional_body_parameters: typing.Dict[str, typing.Any]. A dictionary containing additional parameters to spread into the request's body parameters dict
    """

    timeout_in_seconds: NotRequired[int]
    additional_headers: NotRequired[typing.Dict[str, typing.Any]]
    additional_query_parameters: NotRequired[typing.Dict[str, typing.Any]]
    additional_body_parameters: NotRequired[typing.Dict[str, typing.Any]]



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class FinishReason(str, enum.Enum):
    COMPLETE = "COMPLETE"
    ERROR = "ERROR"
    ERROR_TOXIC = "ERROR_TOXIC"
    ERROR_LIMIT = "ERROR_LIMIT"
    USER_CANCEL = "USER_CANCEL"
    MAX_TOKENS = "MAX_TOKENS"

    def visit(
        self,
        complete: typing.Callable[[], T_Result],
        error: typing.Callable[[], T_Result],
        error_toxic: typing.Callable[[], T_Result],
        error_limit: typing.Callable[[], T_Result],
        user_cancel: typing.Callable[[], T_Result],
        max_tokens: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is FinishReason.COMPLETE:
            return complete()
        if self is FinishReason.ERROR:
            return error()
        if self is FinishReason.ERROR_TOXIC:
            return error_toxic()
        if self is FinishReason.ERROR_LIMIT:
            return error_limit()
        if self is FinishReason.USER_CANCEL:
            return user_cancel()
        if self is FinishReason.MAX_TOKENS:
            return max_tokens()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class ChatCitation(pydantic.BaseModel):
    """
    A section of the generated reply which cites external knowledge.
    """

    start: int = pydantic.Field(
        description="The index of text that the citation starts at, counting from zero. For example, a generation of `Hello, world!` with a citation on `world` would have a start value of `7`. This is because the citation starts at `w`, which is the seventh character."
    )
    end: int = pydantic.Field(
        description="The index of text that the citation ends after, counting from zero. For example, a generation of `Hello, world!` with a citation on `world` would have an end value of `11`. This is because the citation ends after `d`, which is the eleventh character."
    )
    text: str = pydantic.Field(
        description="The text of the citation. For example, a generation of `Hello, world!` with a citation of `world` would have a text value of `world`."
    )
    document_ids: typing.List[str] = pydantic.Field(
        description="Identifiers of documents cited by this section of the generated reply."
    )

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .types import DatasetsCreateResponse, DatasetsGetResponse, DatasetsGetUsageResponse, DatasetsListResponse

__all__ = ["DatasetsCreateResponse", "DatasetsGetResponse", "DatasetsGetUsageResponse", "DatasetsListResponse"]



# This file was auto-generated by Fern from our API Definition.

from . import connectors, datasets, embed_jobs, models
from .datasets import DatasetsCreateResponse, DatasetsGetResponse, DatasetsGetUsageResponse, DatasetsListResponse
from .embed_jobs import CreateEmbedJobRequestTruncate

__all__ = [
    "CreateEmbedJobRequestTruncate",
    "DatasetsCreateResponse",
    "DatasetsGetResponse",
    "DatasetsGetUsageResponse",
    "DatasetsListResponse",
    "connectors",
    "datasets",
    "embed_jobs",
    "models",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime
from .dataset_part import DatasetPart
from .dataset_type import DatasetType
from .dataset_validation_status import DatasetValidationStatus

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Dataset(pydantic.BaseModel):
    id: str = pydantic.Field(description="The dataset ID")
    name: str = pydantic.Field(description="The name of the dataset")
    created_at: dt.datetime = pydantic.Field(description="The creation date")
    updated_at: dt.datetime = pydantic.Field(description="The last update date")
    dataset_type: DatasetType
    validation_status: DatasetValidationStatus
    validation_error: typing.Optional[str] = pydantic.Field(default=None, description="Errors found during validation")
    schema_: typing.Optional[str] = pydantic.Field(
        alias="schema", default=None, description="the avro schema of the dataset"
    )
    required_fields: typing.Optional[typing.List[str]] = None
    preserve_fields: typing.Optional[typing.List[str]] = None
    dataset_parts: typing.Optional[typing.List[DatasetPart]] = pydantic.Field(
        default=None, description="the underlying files that make up the dataset"
    )
    validation_warnings: typing.Optional[typing.List[str]] = pydantic.Field(
        default=None, description="warnings found during validation"
    )

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.jsonable_encoder import jsonable_encoder
from ...core.remove_none_from_dict import remove_none_from_dict
from ...core.request_options import RequestOptions
from ...errors.bad_request_error import BadRequestError
from ...errors.forbidden_error import ForbiddenError
from ...errors.internal_server_error import InternalServerError
from ...errors.not_found_error import NotFoundError
from ...errors.too_many_requests_error import TooManyRequestsError
from ...types.create_connector_o_auth import CreateConnectorOAuth
from ...types.create_connector_response import CreateConnectorResponse
from ...types.create_connector_service_auth import CreateConnectorServiceAuth
from ...types.delete_connector_response import DeleteConnectorResponse
from ...types.get_connector_response import GetConnectorResponse
from ...types.list_connectors_response import ListConnectorsResponse
from ...types.o_auth_authorize_response import OAuthAuthorizeResponse
from ...types.update_connector_response import UpdateConnectorResponse

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ConnectorsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        limit: typing.Optional[float] = None,
        offset: typing.Optional[float] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListConnectorsResponse:
        """
        Returns a list of connectors ordered by descending creation date (newer first). See ['Managing your Connector'](https://docs.cohere.com/docs/managing-your-connector) for more information.

        Parameters:
            - limit: typing.Optional[float]. Maximum number of connectors to return [0, 100].

            - offset: typing.Optional[float]. Number of connectors to skip before returning results [0, inf].

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from cohere.base_client import BaseCohere

        client = BaseCohere(
            client_name="YOUR_CLIENT_NAME",
            token="YOUR_TOKEN",
        )
        client.connectors.list()
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "connectors"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "limit": limit,
                        "offset": offset,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ListConnectorsResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        name: str,
        description: typing.Optional[str] = OMIT,
        url: str,
        excludes: typing.Optional[typing.Sequence[str]] = OMIT,
        oauth: typing.Optional[CreateConnectorOAuth] = OMIT,
        active: typing.Optional[bool] = OMIT,
        continue_on_failure: typing.Optional[bool] = OMIT,
        service_auth: typing.Optional[CreateConnectorServiceAuth] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateConnectorResponse:
        """
        Creates a new connector. The connector is tested during registration and will cancel registration when the test is unsuccessful. See ['Creating and Deploying a Connector'](https://docs.cohere.com/docs/creating-and-deploying-a-connector) for more information.

        Parameters:
            - name: str. A human-readable name for the connector.

            - description: typing.Optional[str]. A description of the connector.

            - url: str. The URL of the connector that will be used to search for documents.

            - excludes: typing.Optional[typing.Sequence[str]]. A list of fields to exclude from the prompt (fields remain in the document).

            - oauth: typing.Optional[CreateConnectorOAuth]. The OAuth 2.0 configuration for the connector. Cannot be specified if service_auth is specified.

            - active: typing.Optional[bool]. Whether the connector is active or not.

            - continue_on_failure: typing.Optional[bool]. Whether a chat request should continue or not if the request to this connector fails.

            - service_auth: typing.Optional[CreateConnectorServiceAuth]. The service to service authentication configuration for the connector. Cannot be specified if oauth is specified.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from cohere.base_client import BaseCohere

        client = BaseCohere(
            client_name="YOUR_CLIENT_NAME",
            token="YOUR_TOKEN",
        )
        client.connectors.create(
            name="name",
            url="url",
        )
        """
        _request: typing.Dict[str, typing.Any] = {"name": name, "url": url}
        if description is not OMIT:
            _request["description"] = description
        if excludes is not OMIT:
            _request["excludes"] = excludes
        if oauth is not OMIT:
            _request["oauth"] = oauth
        if active is not OMIT:
            _request["active"] = active
        if continue_on_failure is not OMIT:
            _request["continue_on_failure"] = continue_on_failure
        if service_auth is not OMIT:
            _request["service_auth"] = service_auth
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "connectors"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(_request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(_request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CreateConnectorResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetConnectorResponse:
        """
        Retrieve a connector by ID. See ['Connectors'](https://docs.cohere.com/docs/connectors) for more information.

        Parameters:
            - id: str. The ID of the connector to retrieve.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from cohere.base_client import BaseCohere

        client = BaseCohere(
            client_name="YOUR_CLIENT_NAME",
            token="YOUR_TOKEN",
        )
        client.connectors.get(
            id="id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"connectors/{jsonable_encoder(id)}"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetConnectorResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> DeleteConnectorResponse:
        """
        Delete a connector by ID. See ['Connectors'](https://docs.cohere.com/docs/connectors) for more information.

        Parameters:
            - id: str. The ID of the connector to delete.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from cohere.base_client import BaseCohere

        client = BaseCohere(
            client_name="YOUR_CLIENT_NAME",
            token="YOUR_TOKEN",
        )
        client.connectors.delete(
            id="id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"connectors/{jsonable_encoder(id)}"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(DeleteConnectorResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        url: typing.Optional[str] = OMIT,
        excludes: typing.Optional[typing.Sequence[str]] = OMIT,
        oauth: typing.Optional[CreateConnectorOAuth] = OMIT,
        active: typing.Optional[bool] = OMIT,
        continue_on_failure: typing.Optional[bool] = OMIT,
        service_auth: typing.Optional[CreateConnectorServiceAuth] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateConnectorResponse:
        """
        Update a connector by ID. Omitted fields will not be updated. See ['Managing your Connector'](https://docs.cohere.com/docs/managing-your-connector) for more information.

        Parameters:
            - id: str. The ID of the connector to update.

            - name: typing.Optional[str]. A human-readable name for the connector.

            - url: typing.Optional[str]. The URL of the connector that will be used to search for documents.

            - excludes: typing.Optional[typing.Sequence[str]]. A list of fields to exclude from the prompt (fields remain in the document).

            - oauth: typing.Optional[CreateConnectorOAuth]. The OAuth 2.0 configuration for the connector. Cannot be specified if service_auth is specified.

            - active: typing.Optional[bool].

            - continue_on_failure: typing.Optional[bool].

            - service_auth: typing.Optional[CreateConnectorServiceAuth]. The service to service authentication configuration for the connector. Cannot be specified if oauth is specified.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from cohere.base_client import BaseCohere

        client = BaseCohere(
            client_name="YOUR_CLIENT_NAME",
            token="YOUR_TOKEN",
        )
        client.connectors.update(
            id="id",
        )
        """
        _request: typing.Dict[str, typing.Any] = {}
        if name is not OMIT:
            _request["name"] = name
        if url is not OMIT:
            _request["url"] = url
        if excludes is not OMIT:
            _request["excludes"] = excludes
        if oauth is not OMIT:
            _request["oauth"] = oauth
        if active is not OMIT:
            _request["active"] = active
        if continue_on_failure is not OMIT:
            _request["continue_on_failure"] = continue_on_failure
        if service_auth is not OMIT:
            _request["service_auth"] = service_auth
        _response = self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"connectors/{jsonable_encoder(id)}"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(_request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(_request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(UpdateConnectorResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def o_auth_authorize(
        self,
        id: str,
        *,
        after_token_redirect: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> OAuthAuthorizeResponse:
        """
        Authorize the connector with the given ID for the connector oauth app. See ['Connector Authentication'](https://docs.cohere.com/docs/connector-authentication) for more information.

        Parameters:
            - id: str. The ID of the connector to authorize.

            - after_token_redirect: typing.Optional[str]. The URL to redirect to after the connector has been authorized.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from cohere.base_client import BaseCohere

        client = BaseCohere(
            client_name="YOUR_CLIENT_NAME",
            token="YOUR_TOKEN",
        )
        client.connectors.o_auth_authorize(
            id="id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"connectors/{jsonable_encoder(id)}/oauth/authorize"
            ),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "after_token_redirect": after_token_redirect,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            json=jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))
            if request_options is not None
            else None,
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(OAuthAuthorizeResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncConnectorsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        limit: typing.Optional[float] = None,
        offset: typing.Optional[float] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListConnectorsResponse:
        """
        Returns a list of connectors ordered by descending creation date (newer first). See ['Managing your Connector'](https://docs.cohere.com/docs/managing-your-connector) for more information.

        Parameters:
            - limit: typing.Optional[float]. Maximum number of connectors to return [0, 100].

            - offset: typing.Optional[float]. Number of connectors to skip before returning results [0, inf].

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from cohere.base_client import AsyncBaseCohere

        client = AsyncBaseCohere(
            client_name="YOUR_CLIENT_NAME",
            token="YOUR_TOKEN",
        )
        await client.connectors.list()
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "connectors"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "limit": limit,
                        "offset": offset,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ListConnectorsResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        name: str,
        description: typing.Optional[str] = OMIT,
        url: str,
        excludes: typing.Optional[typing.Sequence[str]] = OMIT,
        oauth: typing.Optional[CreateConnectorOAuth] = OMIT,
        active: typing.Optional[bool] = OMIT,
        continue_on_failure: typing.Optional[bool] = OMIT,
        service_auth: typing.Optional[CreateConnectorServiceAuth] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateConnectorResponse:
        """
        Creates a new connector. The connector is tested during registration and will cancel registration when the test is unsuccessful. See ['Creating and Deploying a Connector'](https://docs.cohere.com/docs/creating-and-deploying-a-connector) for more information.

        Parameters:
            - name: str. A human-readable name for the connector.

            - description: typing.Optional[str]. A description of the connector.

            - url: str. The URL of the connector that will be used to search for documents.

            - excludes: typing.Optional[typing.Sequence[str]]. A list of fields to exclude from the prompt (fields remain in the document).

            - oauth: typing.Optional[CreateConnectorOAuth]. The OAuth 2.0 configuration for the connector. Cannot be specified if service_auth is specified.

            - active: typing.Optional[bool]. Whether the connector is active or not.

            - continue_on_failure: typing.Optional[bool]. Whether a chat request should continue or not if the request to this connector fails.

            - service_auth: typing.Optional[CreateConnectorServiceAuth]. The service to service authentication configuration for the connector. Cannot be specified if oauth is specified.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from cohere.base_client import AsyncBaseCohere

        client = AsyncBaseCohere(
            client_name="YOUR_CLIENT_NAME",
            token="YOUR_TOKEN",
        )
        await client.connectors.create(
            name="name",
            url="url",
        )
        """
        _request: typing.Dict[str, typing.Any] = {"name": name, "url": url}
        if description is not OMIT:
            _request["description"] = description
        if excludes is not OMIT:
            _request["excludes"] = excludes
        if oauth is not OMIT:
            _request["oauth"] = oauth
        if active is not OMIT:
            _request["active"] = active
        if continue_on_failure is not OMIT:
            _request["continue_on_failure"] = continue_on_failure
        if service_auth is not OMIT:
            _request["service_auth"] = service_auth
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "connectors"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(_request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(_request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CreateConnectorResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetConnectorResponse:
        """
        Retrieve a connector by ID. See ['Connectors'](https://docs.cohere.com/docs/connectors) for more information.

        Parameters:
            - id: str. The ID of the connector to retrieve.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from cohere.base_client import AsyncBaseCohere

        client = AsyncBaseCohere(
            client_name="YOUR_CLIENT_NAME",
            token="YOUR_TOKEN",
        )
        await client.connectors.get(
            id="id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"connectors/{jsonable_encoder(id)}"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetConnectorResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteConnectorResponse:
        """
        Delete a connector by ID. See ['Connectors'](https://docs.cohere.com/docs/connectors) for more information.

        Parameters:
            - id: str. The ID of the connector to delete.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from cohere.base_client import AsyncBaseCohere

        client = AsyncBaseCohere(
            client_name="YOUR_CLIENT_NAME",
            token="YOUR_TOKEN",
        )
        await client.connectors.delete(
            id="id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"connectors/{jsonable_encoder(id)}"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(DeleteConnectorResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        url: typing.Optional[str] = OMIT,
        excludes: typing.Optional[typing.Sequence[str]] = OMIT,
        oauth: typing.Optional[CreateConnectorOAuth] = OMIT,
        active: typing.Optional[bool] = OMIT,
        continue_on_failure: typing.Optional[bool] = OMIT,
        service_auth: typing.Optional[CreateConnectorServiceAuth] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateConnectorResponse:
        """
        Update a connector by ID. Omitted fields will not be updated. See ['Managing your Connector'](https://docs.cohere.com/docs/managing-your-connector) for more information.

        Parameters:
            - id: str. The ID of the connector to update.

            - name: typing.Optional[str]. A human-readable name for the connector.

            - url: typing.Optional[str]. The URL of the connector that will be used to search for documents.

            - excludes: typing.Optional[typing.Sequence[str]]. A list of fields to exclude from the prompt (fields remain in the document).

            - oauth: typing.Optional[CreateConnectorOAuth]. The OAuth 2.0 configuration for the connector. Cannot be specified if service_auth is specified.

            - active: typing.Optional[bool].

            - continue_on_failure: typing.Optional[bool].

            - service_auth: typing.Optional[CreateConnectorServiceAuth]. The service to service authentication configuration for the connector. Cannot be specified if oauth is specified.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from cohere.base_client import AsyncBaseCohere

        client = AsyncBaseCohere(
            client_name="YOUR_CLIENT_NAME",
            token="YOUR_TOKEN",
        )
        await client.connectors.update(
            id="id",
        )
        """
        _request: typing.Dict[str, typing.Any] = {}
        if name is not OMIT:
            _request["name"] = name
        if url is not OMIT:
            _request["url"] = url
        if excludes is not OMIT:
            _request["excludes"] = excludes
        if oauth is not OMIT:
            _request["oauth"] = oauth
        if active is not OMIT:
            _request["active"] = active
        if continue_on_failure is not OMIT:
            _request["continue_on_failure"] = continue_on_failure
        if service_auth is not OMIT:
            _request["service_auth"] = service_auth
        _response = await self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"connectors/{jsonable_encoder(id)}"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(_request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(_request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(UpdateConnectorResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def o_auth_authorize(
        self,
        id: str,
        *,
        after_token_redirect: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> OAuthAuthorizeResponse:
        """
        Authorize the connector with the given ID for the connector oauth app. See ['Connector Authentication'](https://docs.cohere.com/docs/connector-authentication) for more information.

        Parameters:
            - id: str. The ID of the connector to authorize.

            - after_token_redirect: typing.Optional[str]. The URL to redirect to after the connector has been authorized.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from cohere.base_client import AsyncBaseCohere

        client = AsyncBaseCohere(
            client_name="YOUR_CLIENT_NAME",
            token="YOUR_TOKEN",
        )
        await client.connectors.o_auth_authorize(
            id="id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"connectors/{jsonable_encoder(id)}/oauth/authorize"
            ),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "after_token_redirect": after_token_redirect,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            json=jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))
            if request_options is not None
            else None,
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(OAuthAuthorizeResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class ChatConnector(pydantic.BaseModel):
    """
    The connector used for fetching documents.
    """

    id: str = pydantic.Field(description="The identifier of the connector.")
    user_access_token: typing.Optional[str] = pydantic.Field(
        default=None,
        description="When specified, this user access token will be passed to the connector in the Authorization header instead of the Cohere generated one.",
    )
    continue_on_failure: typing.Optional[bool] = pydantic.Field(
        default=None,
        description=(
            "Defaults to `false`.\n"
            "\n"
            "When `true`, the request will continue if this connector returned an error.\n"
        ),
    )
    options: typing.Optional[typing.Dict[str, typing.Any]] = pydantic.Field(
        default=None,
        description=(
            "Provides the connector with different settings at request time. The key/value pairs of this object are specific to each connector.\n"
            "\n"
            "For example, the connector `web-search` supports the `site` option, which limits search results to the specified domain.\n"
        ),
    )

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ChatRequestCitationQuality(str, enum.Enum):
    """
    Defaults to `"accurate"`.

    Dictates the approach taken to generating citations as part of the RAG flow by allowing the user to specify whether they want `"accurate"` results or `"fast"` results.
    """

    FAST = "fast"
    ACCURATE = "accurate"

    def visit(self, fast: typing.Callable[[], T_Result], accurate: typing.Callable[[], T_Result]) -> T_Result:
        if self is ChatRequestCitationQuality.FAST:
            return fast()
        if self is ChatRequestCitationQuality.ACCURATE:
            return accurate()



# This file was auto-generated by Fern from our API Definition.

import typing

import httpx


class BaseClientWrapper:
    def __init__(
        self,
        *,
        client_name: typing.Optional[str] = None,
        token: typing.Union[str, typing.Callable[[], str]],
        base_url: str,
    ):
        self._client_name = client_name
        self._token = token
        self._base_url = base_url

    def get_headers(self) -> typing.Dict[str, str]:
        headers: typing.Dict[str, str] = {
            "X-Fern-Language": "Python",
            "X-Fern-SDK-Name": "cohere",
            "X-Fern-SDK-Version": "5.0.0a8",
        }
        if self._client_name is not None:
            headers["X-Client-Name"] = self._client_name
        headers["Authorization"] = f"Bearer {self._get_token()}"
        return headers

    def _get_token(self) -> str:
        if isinstance(self._token, str):
            return self._token
        else:
            return self._token()

    def get_base_url(self) -> str:
        return self._base_url


class SyncClientWrapper(BaseClientWrapper):
    def __init__(
        self,
        *,
        client_name: typing.Optional[str] = None,
        token: typing.Union[str, typing.Callable[[], str]],
        base_url: str,
        httpx_client: httpx.Client,
    ):
        super().__init__(client_name=client_name, token=token, base_url=base_url)
        self.httpx_client = httpx_client


class AsyncClientWrapper(BaseClientWrapper):
    def __init__(
        self,
        *,
        client_name: typing.Optional[str] = None,
        token: typing.Union[str, typing.Callable[[], str]],
        base_url: str,
        httpx_client: httpx.AsyncClient,
    ):
        super().__init__(client_name=client_name, token=token, base_url=base_url)
        self.httpx_client = httpx_client



# This file was auto-generated by Fern from our API Definition.

from .types import (
    ApiMeta,
    ApiMetaApiVersion,
    ApiMetaBilledUnits,
    AuthTokenType,
    ChatCitation,
    ChatCitationGenerationEvent,
    ChatConnector,
    ChatDocument,
    ChatMessage,
    ChatMessageRole,
    ChatRequestCitationQuality,
    ChatRequestPromptOverride,
    ChatRequestPromptTruncation,
    ChatRequestSearchOptions,
    ChatRequestToolResultsItem,
    ChatSearchQueriesGenerationEvent,
    ChatSearchQuery,
    ChatSearchResult,
    ChatSearchResultConnector,
    ChatSearchResultsEvent,
    ChatStreamEndEvent,
    ChatStreamEndEventFinishReason,
    ChatStreamEvent,
    ChatStreamRequestCitationQuality,
    ChatStreamRequestPromptOverride,
    ChatStreamRequestPromptTruncation,
    ChatStreamRequestSearchOptions,
    ChatStreamRequestToolResultsItem,
    ChatStreamStartEvent,
    ChatTextGenerationEvent,
    ChatToolInputsGenerationEvent,
    ClassifyExample,
    ClassifyRequestTruncate,
    ClassifyResponse,
    ClassifyResponseClassificationsItem,
    ClassifyResponseClassificationsItemClassificationType,
    ClassifyResponseClassificationsItemLabelsValue,
    CompatibleEndpoint,
    Connector,
    ConnectorAuthStatus,
    ConnectorOAuth,
    CreateConnectorOAuth,
    CreateConnectorResponse,
    CreateConnectorServiceAuth,
    CreateEmbedJobResponse,
    Dataset,
    DatasetPart,
    DatasetType,
    DatasetValidationStatus,
    DeleteConnectorResponse,
    DetokenizeResponse,
    EmbedByTypeResponse,
    EmbedByTypeResponseEmbeddings,
    EmbedFloatsResponse,
    EmbedInputType,
    EmbedJob,
    EmbedJobStatus,
    EmbedJobTruncate,
    EmbedRequestEmbeddingTypesItem,
    EmbedRequestTruncate,
    EmbedResponse,
    EmbedResponse_EmbeddingsByType,
    EmbedResponse_EmbeddingsFloats,
    FinishReason,
    GenerateRequestReturnLikelihoods,
    GenerateRequestTruncate,
    GenerateStreamEnd,
    GenerateStreamEndResponse,
    GenerateStreamError,
    GenerateStreamEvent,
    GenerateStreamRequestReturnLikelihoods,
    GenerateStreamRequestTruncate,
    GenerateStreamText,
    GenerateStreamedResponse,
    GenerateStreamedResponse_StreamEnd,
    GenerateStreamedResponse_StreamError,
    GenerateStreamedResponse_TextGeneration,
    Generation,
    GetConnectorResponse,
    ListConnectorsResponse,
    ListEmbedJobResponse,
    ListModelsResponse,
    Model,
    NonStreamedChatResponse,
    OAuthAuthorizeResponse,
    ParseInfo,
    RerankRequestDocumentsItem,
    RerankRequestDocumentsItemText,
    RerankResponse,
    RerankResponseResultsItem,
    RerankResponseResultsItemDocument,
    SingleGeneration,
    SingleGenerationInStream,
    SingleGenerationTokenLikelihoodsItem,
    StreamedChatResponse,
    StreamedChatResponse_CitationGeneration,
    StreamedChatResponse_SearchQueriesGeneration,
    StreamedChatResponse_SearchResults,
    StreamedChatResponse_StreamEnd,
    StreamedChatResponse_StreamStart,
    StreamedChatResponse_TextGeneration,
    StreamedChatResponse_ToolInputsGeneration,
    SummarizeRequestExtractiveness,
    SummarizeRequestFormat,
    SummarizeRequestLength,
    SummarizeResponse,
    TokenizeResponse,
    Tool,
    ToolInput,
    ToolParameterDefinitionsValue,
    UpdateConnectorResponse,
)
from .errors import BadRequestError, ForbiddenError, InternalServerError, NotFoundError, TooManyRequestsError
from .resources import (
    CreateEmbedJobRequestTruncate,
    DatasetsCreateResponse,
    DatasetsGetResponse,
    DatasetsGetUsageResponse,
    DatasetsListResponse,
    connectors,
    datasets,
    embed_jobs,
    models,
)
from .client import AsyncClient, Client
from .environment import BaseCohereEnvironment

__all__ = [
    "ApiMeta",
    "ApiMetaApiVersion",
    "ApiMetaBilledUnits",
    "AsyncClient",
    "AuthTokenType",
    "BadRequestError",
    "BaseCohereEnvironment",
    "ChatCitation",
    "ChatCitationGenerationEvent",
    "ChatConnector",
    "ChatDocument",
    "ChatMessage",
    "ChatMessageRole",
    "ChatRequestCitationQuality",
    "ChatRequestPromptOverride",
    "ChatRequestPromptTruncation",
    "ChatRequestSearchOptions",
    "ChatRequestToolResultsItem",
    "ChatSearchQueriesGenerationEvent",
    "ChatSearchQuery",
    "ChatSearchResult",
    "ChatSearchResultConnector",
    "ChatSearchResultsEvent",
    "ChatStreamEndEvent",
    "ChatStreamEndEventFinishReason",
    "ChatStreamEvent",
    "ChatStreamRequestCitationQuality",
    "ChatStreamRequestPromptOverride",
    "ChatStreamRequestPromptTruncation",
    "ChatStreamRequestSearchOptions",
    "ChatStreamRequestToolResultsItem",
    "ChatStreamStartEvent",
    "ChatTextGenerationEvent",
    "ChatToolInputsGenerationEvent",
    "ClassifyExample",
    "ClassifyRequestTruncate",
    "ClassifyResponse",
    "ClassifyResponseClassificationsItem",
    "ClassifyResponseClassificationsItemClassificationType",
    "ClassifyResponseClassificationsItemLabelsValue",
    "Client",
    "CompatibleEndpoint",
    "Connector",
    "ConnectorAuthStatus",
    "ConnectorOAuth",
    "CreateConnectorOAuth",
    "CreateConnectorResponse",
    "CreateConnectorServiceAuth",
    "CreateEmbedJobRequestTruncate",
    "CreateEmbedJobResponse",
    "Dataset",
    "DatasetPart",
    "DatasetType",
    "DatasetValidationStatus",
    "DatasetsCreateResponse",
    "DatasetsGetResponse",
    "DatasetsGetUsageResponse",
    "DatasetsListResponse",
    "DeleteConnectorResponse",
    "DetokenizeResponse",
    "EmbedByTypeResponse",
    "EmbedByTypeResponseEmbeddings",
    "EmbedFloatsResponse",
    "EmbedInputType",
    "EmbedJob",
    "EmbedJobStatus",
    "EmbedJobTruncate",
    "EmbedRequestEmbeddingTypesItem",
    "EmbedRequestTruncate",
    "EmbedResponse",
    "EmbedResponse_EmbeddingsByType",
    "EmbedResponse_EmbeddingsFloats",
    "FinishReason",
    "ForbiddenError",
    "GenerateRequestReturnLikelihoods",
    "GenerateRequestTruncate",
    "GenerateStreamEnd",
    "GenerateStreamEndResponse",
    "GenerateStreamError",
    "GenerateStreamEvent",
    "GenerateStreamRequestReturnLikelihoods",
    "GenerateStreamRequestTruncate",
    "GenerateStreamText",
    "GenerateStreamedResponse",
    "GenerateStreamedResponse_StreamEnd",
    "GenerateStreamedResponse_StreamError",
    "GenerateStreamedResponse_TextGeneration",
    "Generation",
    "GetConnectorResponse",
    "InternalServerError",
    "ListConnectorsResponse",
    "ListEmbedJobResponse",
    "ListModelsResponse",
    "Model",
    "NonStreamedChatResponse",
    "NotFoundError",
    "OAuthAuthorizeResponse",
    "ParseInfo",
    "RerankRequestDocumentsItem",
    "RerankRequestDocumentsItemText",
    "RerankResponse",
    "RerankResponseResultsItem",
    "RerankResponseResultsItemDocument",
    "SingleGeneration",
    "SingleGenerationInStream",
    "SingleGenerationTokenLikelihoodsItem",
    "StreamedChatResponse",
    "StreamedChatResponse_CitationGeneration",
    "StreamedChatResponse_SearchQueriesGeneration",
    "StreamedChatResponse_SearchResults",
    "StreamedChatResponse_StreamEnd",
    "StreamedChatResponse_StreamStart",
    "StreamedChatResponse_TextGeneration",
    "StreamedChatResponse_ToolInputsGeneration",
    "SummarizeRequestExtractiveness",
    "SummarizeRequestFormat",
    "SummarizeRequestLength",
    "SummarizeResponse",
    "TokenizeResponse",
    "TooManyRequestsError",
    "Tool",
    "ToolInput",
    "ToolParameterDefinitionsValue",
    "UpdateConnectorResponse",
    "connectors",
    "datasets",
    "embed_jobs",
    "models",
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from .embed_by_type_response import EmbedByTypeResponse
from .embed_floats_response import EmbedFloatsResponse


class EmbedResponse_EmbeddingsFloats(EmbedFloatsResponse):
    response_type: typing.Literal["embeddings_floats"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class EmbedResponse_EmbeddingsByType(EmbedByTypeResponse):
    response_type: typing.Literal["embeddings_by_type"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


EmbedResponse = typing.Union[EmbedResponse_EmbeddingsFloats, EmbedResponse_EmbeddingsByType]



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class EmbedRequestEmbeddingTypesItem(str, enum.Enum):
    FLOAT = "float"
    INT_8 = "int8"
    UINT_8 = "uint8"
    BINARY = "binary"
    UBINARY = "ubinary"

    def visit(
        self,
        float_: typing.Callable[[], T_Result],
        int_8: typing.Callable[[], T_Result],
        uint_8: typing.Callable[[], T_Result],
        binary: typing.Callable[[], T_Result],
        ubinary: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is EmbedRequestEmbeddingTypesItem.FLOAT:
            return float_()
        if self is EmbedRequestEmbeddingTypesItem.INT_8:
            return int_8()
        if self is EmbedRequestEmbeddingTypesItem.UINT_8:
            return uint_8()
        if self is EmbedRequestEmbeddingTypesItem.BINARY:
            return binary()
        if self is EmbedRequestEmbeddingTypesItem.UBINARY:
            return ubinary()



# This file was auto-generated by Fern from our API Definition.

from .datasets_create_response import DatasetsCreateResponse
from .datasets_get_response import DatasetsGetResponse
from .datasets_get_usage_response import DatasetsGetUsageResponse
from .datasets_list_response import DatasetsListResponse

__all__ = ["DatasetsCreateResponse", "DatasetsGetResponse", "DatasetsGetUsageResponse", "DatasetsListResponse"]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ... import core
from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.datetime_utils import serialize_datetime
from ...core.jsonable_encoder import jsonable_encoder
from ...core.remove_none_from_dict import remove_none_from_dict
from ...core.request_options import RequestOptions
from ...errors.too_many_requests_error import TooManyRequestsError
from ...types.dataset_type import DatasetType
from .types.datasets_create_response import DatasetsCreateResponse
from .types.datasets_get_response import DatasetsGetResponse
from .types.datasets_get_usage_response import DatasetsGetUsageResponse
from .types.datasets_list_response import DatasetsListResponse

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class DatasetsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        dataset_type: typing.Optional[str] = None,
        before: typing.Optional[dt.datetime] = None,
        after: typing.Optional[dt.datetime] = None,
        limit: typing.Optional[str] = None,
        offset: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DatasetsListResponse:
        """
        List datasets that have been created.

        Parameters:
            - dataset_type: typing.Optional[str]. optional filter by dataset type

            - before: typing.Optional[dt.datetime]. optional filter before a date

            - after: typing.Optional[dt.datetime]. optional filter after a date

            - limit: typing.Optional[str]. optional limit to number of results

            - offset: typing.Optional[str]. optional offset to start of results

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from cohere.base_client import BaseCohere

        client = BaseCohere(
            client_name="YOUR_CLIENT_NAME",
            token="YOUR_TOKEN",
        )
        client.datasets.list()
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "datasets"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "datasetType": dataset_type,
                        "before": serialize_datetime(before) if before is not None else None,
                        "after": serialize_datetime(after) if after is not None else None,
                        "limit": limit,
                        "offset": offset,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(DatasetsListResponse, _response.json())  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        name: typing.Optional[str] = None,
        type: typing.Optional[DatasetType] = None,
        keep_original_file: typing.Optional[bool] = None,
        skip_malformed_input: typing.Optional[bool] = None,
        keep_fields: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        optional_fields: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        text_separator: typing.Optional[str] = None,
        csv_delimiter: typing.Optional[str] = None,
        data: core.File,
        eval_data: typing.Optional[core.File] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DatasetsCreateResponse:
        """
        Create a dataset by uploading a file. See ['Dataset Creation'](https://docs.cohere.com/docs/datasets#dataset-creation) for more information.

        Parameters:
            - name: typing.Optional[str]. The name of the uploaded dataset.

            - type: typing.Optional[DatasetType]. The dataset type, which is used to validate the data.

            - keep_original_file: typing.Optional[bool]. Indicates if the original file should be stored.

            - skip_malformed_input: typing.Optional[bool]. Indicates whether rows with malformed input should be dropped (instead of failing the validation check). Dropped rows will be returned in the warnings field.

            - keep_fields: typing.Optional[typing.Union[str, typing.Sequence[str]]]. List of names of fields that will be persisted in the Dataset. By default the Dataset will retain only the required fields indicated in the [schema for the corresponding Dataset type](https://docs.cohere.com/docs/datasets#dataset-types). For example, datasets of type `embed-input` will drop all fields other than the required `text` field. If any of the fields in `keep_fields` are missing from the uploaded file, Dataset validation will fail.

            - optional_fields: typing.Optional[typing.Union[str, typing.Sequence[str]]]. List of names of fields that will be persisted in the Dataset. By default the Dataset will retain only the required fields indicated in the [schema for the corresponding Dataset type](https://docs.cohere.com/docs/datasets#dataset-types). For example, Datasets of type `embed-input` will drop all fields other than the required `text` field. If any of the fields in `optional_fields` are missing from the uploaded file, Dataset validation will pass.

            - text_separator: typing.Optional[str]. Raw .txt uploads will be split into entries using the text_separator value.

            - csv_delimiter: typing.Optional[str]. The delimiter used for .csv uploads.

            - data: core.File. See core.File for more documentation

            - eval_data: typing.Optional[core.File]. See core.File for more documentation

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "datasets"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "name": name,
                        "type": type,
                        "keep_original_file": keep_original_file,
                        "skip_malformed_input": skip_malformed_input,
                        "keep_fields": keep_fields,
                        "optional_fields": optional_fields,
                        "text_separator": text_separator,
                        "csv_delimiter": csv_delimiter,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            data=jsonable_encoder(remove_none_from_dict({}))
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(remove_none_from_dict({})),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            files=core.convert_file_dict_to_httpx_tuples(remove_none_from_dict({"data": data, "eval_data": eval_data})),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(DatasetsCreateResponse, _response.json())  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_usage(self, *, request_options: typing.Optional[RequestOptions] = None) -> DatasetsGetUsageResponse:
        """
        View the dataset storage usage for your Organization. Each Organization can have up to 10GB of storage across all their users.

        Parameters:
            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from cohere.base_client import BaseCohere

        client = BaseCohere(
            client_name="YOUR_CLIENT_NAME",
            token="YOUR_TOKEN",
        )
        client.datasets.get_usage()
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "datasets/usage"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(DatasetsGetUsageResponse, _response.json())  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> DatasetsGetResponse:
        """
        Retrieve a dataset by ID. See ['Datasets'](https://docs.cohere.com/docs/datasets) for more information.

        Parameters:
            - id: str.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from cohere.base_client import BaseCohere

        client = BaseCohere(
            client_name="YOUR_CLIENT_NAME",
            token="YOUR_TOKEN",
        )
        client.datasets.get(
            id="id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"datasets/{jsonable_encoder(id)}"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(DatasetsGetResponse, _response.json())  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Dict[str, typing.Any]:
        """
        Delete a dataset by ID. Datasets are automatically deleted after 30 days, but they can also be deleted manually.

        Parameters:
            - id: str.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from cohere.base_client import BaseCohere

        client = BaseCohere(
            client_name="YOUR_CLIENT_NAME",
            token="YOUR_TOKEN",
        )
        client.datasets.delete(
            id="id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"datasets/{jsonable_encoder(id)}"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.Dict[str, typing.Any], _response.json())  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncDatasetsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        dataset_type: typing.Optional[str] = None,
        before: typing.Optional[dt.datetime] = None,
        after: typing.Optional[dt.datetime] = None,
        limit: typing.Optional[str] = None,
        offset: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DatasetsListResponse:
        """
        List datasets that have been created.

        Parameters:
            - dataset_type: typing.Optional[str]. optional filter by dataset type

            - before: typing.Optional[dt.datetime]. optional filter before a date

            - after: typing.Optional[dt.datetime]. optional filter after a date

            - limit: typing.Optional[str]. optional limit to number of results

            - offset: typing.Optional[str]. optional offset to start of results

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from cohere.base_client import AsyncBaseCohere

        client = AsyncBaseCohere(
            client_name="YOUR_CLIENT_NAME",
            token="YOUR_TOKEN",
        )
        await client.datasets.list()
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "datasets"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "datasetType": dataset_type,
                        "before": serialize_datetime(before) if before is not None else None,
                        "after": serialize_datetime(after) if after is not None else None,
                        "limit": limit,
                        "offset": offset,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(DatasetsListResponse, _response.json())  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        name: typing.Optional[str] = None,
        type: typing.Optional[DatasetType] = None,
        keep_original_file: typing.Optional[bool] = None,
        skip_malformed_input: typing.Optional[bool] = None,
        keep_fields: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        optional_fields: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        text_separator: typing.Optional[str] = None,
        csv_delimiter: typing.Optional[str] = None,
        data: core.File,
        eval_data: typing.Optional[core.File] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DatasetsCreateResponse:
        """
        Create a dataset by uploading a file. See ['Dataset Creation'](https://docs.cohere.com/docs/datasets#dataset-creation) for more information.

        Parameters:
            - name: typing.Optional[str]. The name of the uploaded dataset.

            - type: typing.Optional[DatasetType]. The dataset type, which is used to validate the data.

            - keep_original_file: typing.Optional[bool]. Indicates if the original file should be stored.

            - skip_malformed_input: typing.Optional[bool]. Indicates whether rows with malformed input should be dropped (instead of failing the validation check). Dropped rows will be returned in the warnings field.

            - keep_fields: typing.Optional[typing.Union[str, typing.Sequence[str]]]. List of names of fields that will be persisted in the Dataset. By default the Dataset will retain only the required fields indicated in the [schema for the corresponding Dataset type](https://docs.cohere.com/docs/datasets#dataset-types). For example, datasets of type `embed-input` will drop all fields other than the required `text` field. If any of the fields in `keep_fields` are missing from the uploaded file, Dataset validation will fail.

            - optional_fields: typing.Optional[typing.Union[str, typing.Sequence[str]]]. List of names of fields that will be persisted in the Dataset. By default the Dataset will retain only the required fields indicated in the [schema for the corresponding Dataset type](https://docs.cohere.com/docs/datasets#dataset-types). For example, Datasets of type `embed-input` will drop all fields other than the required `text` field. If any of the fields in `optional_fields` are missing from the uploaded file, Dataset validation will pass.

            - text_separator: typing.Optional[str]. Raw .txt uploads will be split into entries using the text_separator value.

            - csv_delimiter: typing.Optional[str]. The delimiter used for .csv uploads.

            - data: core.File. See core.File for more documentation

            - eval_data: typing.Optional[core.File]. See core.File for more documentation

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "datasets"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "name": name,
                        "type": type,
                        "keep_original_file": keep_original_file,
                        "skip_malformed_input": skip_malformed_input,
                        "keep_fields": keep_fields,
                        "optional_fields": optional_fields,
                        "text_separator": text_separator,
                        "csv_delimiter": csv_delimiter,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            data=jsonable_encoder(remove_none_from_dict({}))
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(remove_none_from_dict({})),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            files=core.convert_file_dict_to_httpx_tuples(remove_none_from_dict({"data": data, "eval_data": eval_data})),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(DatasetsCreateResponse, _response.json())  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_usage(self, *, request_options: typing.Optional[RequestOptions] = None) -> DatasetsGetUsageResponse:
        """
        View the dataset storage usage for your Organization. Each Organization can have up to 10GB of storage across all their users.

        Parameters:
            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from cohere.base_client import AsyncBaseCohere

        client = AsyncBaseCohere(
            client_name="YOUR_CLIENT_NAME",
            token="YOUR_TOKEN",
        )
        await client.datasets.get_usage()
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "datasets/usage"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(DatasetsGetUsageResponse, _response.json())  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> DatasetsGetResponse:
        """
        Retrieve a dataset by ID. See ['Datasets'](https://docs.cohere.com/docs/datasets) for more information.

        Parameters:
            - id: str.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from cohere.base_client import AsyncBaseCohere

        client = AsyncBaseCohere(
            client_name="YOUR_CLIENT_NAME",
            token="YOUR_TOKEN",
        )
        await client.datasets.get(
            id="id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"datasets/{jsonable_encoder(id)}"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(DatasetsGetResponse, _response.json())  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Dict[str, typing.Any]:
        """
        Delete a dataset by ID. Datasets are automatically deleted after 30 days, but they can also be deleted manually.

        Parameters:
            - id: str.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from cohere.base_client import AsyncBaseCohere

        client = AsyncBaseCohere(
            client_name="YOUR_CLIENT_NAME",
            token="YOUR_TOKEN",
        )
        await client.datasets.delete(
            id="id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"datasets/{jsonable_encoder(id)}"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.Dict[str, typing.Any], _response.json())  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)
